@@ -8,19 +8,23 @@
          have_ssl_client_renegotiation/0,
          have_ssl_sni/0,
          have_ssl_log_alert/0,
+         have_ssl_handshake/0,
          have_erlang_sendfile/0,
-         have_crypto_strong_rand_bytes/0,
          have_erlang_now/0,
          have_rand/0,
+         have_start_error_logger/0,
 
-         rand_bytes/1,
          unique_triple/0,
          get_time_tuple/0,
          now_secs/0,
          random_seed/3,
          random_uniform/1,
+         connection_information/2,
+         ssl_handshake/2,
+         start_error_logger/0,
 
          generate/1,
+         purge_old_code/0,
          is_generated/0
         ]).
 
@@ -43,14 +47,14 @@ have_ssl_sni() ->
 have_ssl_log_alert() ->
     is_greater_or_equal(erlang:system_info(version), "5.10.3").
 
+%% ssl:ssl_accept/2 is deprecated since release 21 (ERTS >= 10.0)
+have_ssl_handshake() ->
+    is_greater_or_equal(erlang:system_info(version), "10.0").
+
 %% erlang:sendfile/5 is buggy for R15 & R16 releases (ERTS < 6.0)
 have_erlang_sendfile() ->
     is_greater_or_equal(erlang:system_info(version), "6.0").
 
-%% crypto:rand_bytes/1 is deprecated since releases 19 (ERTS >= 8.0)
-have_crypto_strong_rand_bytes() ->
-    lists:member({strong_rand_bytes, 1}, crypto:module_info(exports)).
-
 %% erlang:now/0 is deprecated since releases 18 (ERTS >= 7.0)
 have_erlang_now() ->
     is_less(erlang:system_info(version), "7.0").
@@ -59,32 +63,29 @@ have_erlang_now() ->
 have_rand() ->
     (code:which(rand) /= non_existing).
 
-rand_bytes(N) ->
-    case have_crypto_strong_rand_bytes() of
-        true  -> crypto:strong_rand_bytes(N);
-        false -> (fun crypto:rand_bytes/1)(N)
-    end.
+%% error_logger is legacy as of release 21 (ERTS >= 10.0)
+have_start_error_logger() ->
+    is_greater_or_equal(erlang:system_info(version), "10.0").
 
 unique_triple() ->
     case have_erlang_now() of
         true ->
             (fun erlang:now/0)();
         false ->
-            {erlang:unique_integer([positive]),
-             erlang:unique_integer([positive]),
-             erlang:unique_integer([positive])}
+            F = fun erlang:unique_integer/1,
+            {F([positive]), F([positive]), F([positive])}
     end.
 
 get_time_tuple() ->
     case have_erlang_now() of
         true  -> (fun erlang:now/0)();
-        false -> erlang:timestamp()
+        false -> (fun erlang:timestamp/0)()
     end.
 
 now_secs() ->
     {M,S,_} = case have_erlang_now() of
                   true  -> (fun erlang:now/0)();
-                  false -> erlang:timestamp()
+                  false -> (fun erlang:timestamp/0)()
               end,
     (M*1000000)+S.
 
@@ -100,6 +101,38 @@ random_uniform(N) ->
         false -> (fun random:uniform/1)(N)
     end.
 
+connection_information(Sock, Items) ->
+    case have_ssl_sni() of
+        true  -> (fun ssl:connection_information/2)(Sock, Items);
+        false -> undefined
+    end.
+
+ssl_handshake(Sock, Timeout) ->
+    case have_ssl_handshake() of
+        true -> (fun ssl:handshake/2)(Sock, Timeout);
+        false ->
+            case (fun ssl:ssl_accept/2)(Sock, Timeout) of
+                ok -> {ok, Sock};
+                Error -> Error
+            end
+    end.
+
+start_error_logger() ->
+    case have_start_error_logger() of
+        true ->
+            case (fun logger:get_handler_config/1)(error_logger) of
+                {ok, _} -> ok;
+                {error, _} ->
+                    LoggerArgs = (fun maps:from_list/1)([{level, info},
+                                                         {filter_default, log},
+                                                         {filters, []}]),
+                    (fun logger:add_handler/3)(error_logger, error_logger,
+                                               LoggerArgs)
+            end;
+        false ->
+            ok
+    end.
+
 is_greater         (Vsn1, Vsn2) -> compare_version(Vsn1, Vsn2) == greater.
 is_less            (Vsn1, Vsn2) -> compare_version(Vsn1, Vsn2) == less.
 is_greater_or_equal(Vsn1, Vsn2) -> not is_less(Vsn1, Vsn2).
@@ -112,13 +145,31 @@ compare_version(Vsn1, Vsn2) ->
 
 compare_version1([], []) ->
     equal;
-compare_version1([X|Rest1], [X|Rest2]) ->
-    compare_version1(Rest1, Rest2);
+compare_version1(_X, []) ->
+    greater;
+compare_version1([], _X) ->
+    less;
 compare_version1([X1], [X2]) ->
     %% For last digit ignore everything after the "-", if any
     Y1 = lists:takewhile(fun(X) -> X /= $- end, X1),
     Y2 = lists:takewhile(fun(X) -> X /= $- end, X2),
     compare_digit(Y1, Y2);
+compare_version1([X1], [X2|_]) ->
+    %% For last digit ignore everything after the "-", if any
+    Y1 = lists:takewhile(fun(X) -> X /= $- end, X1),
+    case compare_digit(Y1, X2) of
+        equal -> less;
+        Else  -> Else
+    end;
+compare_version1([X1|_], [X2]) ->
+    %% For last digit ignore everything after the "-", if any
+    Y2 = lists:takewhile(fun(X) -> X /= $- end, X2),
+    case compare_digit(X1, Y2) of
+        equal -> greater;
+        Else  -> Else
+    end;
+compare_version1([X|Rest1], [X|Rest2]) ->
+    compare_version1(Rest1, Rest2);
 compare_version1([X1|Rest1], [X2|Rest2]) ->
     case compare_digit(X1, X2) of
         equal -> compare_version1(Rest1, Rest2);
@@ -140,6 +191,8 @@ compare_digit(X1, X2) ->
 %% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 is_generated() -> false.
 
+purge_old_code() -> not_necessary.
+
 generate(GC) ->
     code:ensure_loaded(crypto),
     code:ensure_loaded(inet),
@@ -152,6 +205,7 @@ generate(GC) ->
             File = filename:join(yaws:tmpdir("/tmp"), "yaws_dynopts.erl"),
             generate1(File);
         _ ->
+            start_error_logger(),
             error_logger:format("Cannot write yaws_dynopts.erl~n"
                                 "Use the default version~n", [])
     end.
@@ -166,6 +220,7 @@ generate1(ModFile) ->
                         {module, ModName} ->
                             ok;
                         {error, What} ->
+                            start_error_logger(),
                             error_logger:format(
                               "Cannot load module '~p': ~p~n"
                               "Use the default version~n",
@@ -173,24 +228,19 @@ generate1(ModFile) ->
                              )
                     end;
                 _ ->
+                    start_error_logger(),
                     error_logger:format("Compilation of '~p' failed: ~p~n"
                                         "Use the default version~n",
                                         [ModFile])
             end;
         {error, Reason} ->
+            start_error_logger(),
             error_logger:format("Cannot write ~p: ~p~n"
                                 "Use the default version~n", [ModFile, Reason])
     end.
 
 write_module(ModFile) ->
-    case file:open(ModFile, [write]) of
-        {ok, Fd} ->
-            io:format(Fd, source(), []),
-            file:close(Fd),
-            ok;
-        {error, Reason} ->
-            {error, Reason}
-    end.
+    file:write_file(ModFile, source()).
 
 compile_options() ->
     [binary, report,
@@ -198,13 +248,10 @@ compile_options() ->
      {d, 'HAVE_SSL_CLIENT_RENEGOTIATION',  have_ssl_client_renegotiation()},
      {d, 'HAVE_SSL_SNI',                   have_ssl_sni()},
      {d, 'HAVE_SSL_LOG_ALERT',             have_ssl_log_alert()},
-     {d, 'HAVE_ERLANG_SENDFILE',           have_erlang_sendfile()}
+     {d, 'HAVE_SSL_HANDSHAKE',             have_ssl_handshake()},
+     {d, 'HAVE_ERLANG_SENDFILE',           have_erlang_sendfile()},
+     {d, 'HAVE_START_ERROR_LOGGER',        have_start_error_logger()}
     ]
-        ++
-        case have_crypto_strong_rand_bytes() of
-            true  -> [{d, 'HAVE_CRYPTO_STRONG_RAND_BYTES'}];
-            false -> []
-        end
         ++
         case have_erlang_now() of
             true  -> [{d, 'HAVE_ERLANG_NOW'}];
@@ -229,40 +276,39 @@ source() ->
            "    have_ssl_client_renegotiation/0,",
            "    have_ssl_sni/0,",
            "    have_ssl_log_alert/0,",
+           "    have_ssl_handshake/0,",
            "    have_erlang_sendfile/0,",
-           "    have_crypto_strong_rand_bytes/0,",
            "    have_erlang_now/0,",
            "    have_rand/0,"
+           "    have_start_error_logger/0,"
            "",
-           "    rand_bytes/1,",
            "    unique_triple/0,",
            "    get_time_tuple/0,",
            "    now_secs/0,",
            "    random_seed/3,",
            "    random_uniform/1,",
+           "    connection_information/2,",
+           "    ssl_handshake/2,"
+           "    start_error_logger/0,",
            "",
            "    generate/1,",
+           "    purge_old_code/0,",
            "    is_generated/0",
            "   ]).",
            "",
            "",
            "generate(_) -> ok.",
+           "purge_old_code() -> code:soft_purge(?MODULE).",
            "is_generated() -> true.",
            "",
            "have_ssl_honor_cipher_order()   -> ?HAVE_SSL_HONOR_CIPHER_ORDER.",
            "have_ssl_client_renegotiation() -> ?HAVE_SSL_CLIENT_RENEGOTIATION.",
            "have_ssl_sni()                  -> ?HAVE_SSL_SNI.",
            "have_ssl_log_alert()            -> ?HAVE_SSL_LOG_ALERT.",
+           "have_ssl_handshake()            -> ?HAVE_SSL_HANDSHAKE.",
            "have_erlang_sendfile()          -> ?HAVE_ERLANG_SENDFILE.",
+           "have_start_error_logger()       -> ?HAVE_START_ERROR_LOGGER.",
            "",
-           "-ifdef(HAVE_CRYPTO_STRONG_RAND_BYTES).",
-           "have_crypto_strong_rand_bytes() -> true.",
-           "rand_bytes(N) -> crypto:strong_rand_bytes(N).",
-           "-else.",
-           "have_crypto_strong_rand_bytes() -> false.",
-           "rand_bytes(N) -> crypto:rand_bytes(N).",
-           "-endif.",
-           ""
            "-ifdef(HAVE_ERLANG_NOW).",
            "have_erlang_now() -> true.",
            "unique_triple() ->",
@@ -294,6 +340,35 @@ source() ->
            "random_seed(A,B,C) -> random:seed(A,B,C).",
            "random_uniform(N)  -> random:uniform(N).",
            "-endif.",
-           ""
+           "",
+           "-ifdef(HAVE_SSL_SNI).",
+           "connection_information(Sock, Items) -> ",
+           "    ssl:connection_information(Sock, Items).",
+           "-else.",
+           "connection_information(_, _) -> undefined.",
+           "-endif.",
+           "",
+           "-ifdef(HAVE_SSL_HANDSHAKE).",
+           "ssl_handshake(Sock, Timeout) ->",
+           "    ssl:handshake(Sock, Timeout).",
+           "-else.",
+           "ssl_handshake(Sock, Timeout) ->",
+           "    case ssl:ssl_accept(Sock, Timeout) of"
+           "        ok -> {ok, Sock};",
+           "        Error -> Error",
+           "    end.",
+           "-endif.",
+           "-ifdef(HAVE_ERROR_LOGGER_START).",
+           "start_error_logger() ->",
+           "    case logger:get_handler_config(error_logger) of",
+           "        {ok, _} -> ok;",
+           "        {error, _} ->",
+           "            logger:add_handler(error_logger, error_logger,",
+           "                               #{level => info, filter_default => log,"
+           "                                 filters => []})",
+           "    end.",
+           "-else.",
+           "start_error_logger() -> ok.",
+           "-endif."
           ],
     string:join(Src, "\n").
